Package: clustSIGNAL
Type: Package
Title: ClustSIGNAL: a spatial clustering method
Version: 1.4.0
Authors@R: c(
    person("Pratibha", "Panwar", email = "pratibhapanwar.4@gmail.com", 
    role = c("cre", "aut", "ctb"), comment = c(ORCID = "0000-0002-7437-7084")),
    person("Boyi", "Guo", email = "boyi.guo.work@gmail.com", role = "aut"),
    person("Haowen", "Zhao", email = "haowen.zhou19@gmail.com", role = "aut"),
    person("Stephanie", "Hicks", email = "shicks19@jhu.edu", role = "aut"),
    person("Shila", "Ghazanfar", email = "shazanfar@gmail.com", 
    role = c("aut", "ctb"), comment = c(ORCID = "0000-0001-7861-6997")))
Description: clustSIGNAL: clustering of Spatially Informed Gene expression with 
    Neighbourhood Adapted Learning. A tool for adaptively smoothing and clustering 
    gene expression data. clustSIGNAL uses entropy to measure heterogeneity of 
    cell neighbourhoods and performs a weighted, adaptive smoothing, where 
    homogeneous neighbourhoods are smoothed more and heterogeneous neighbourhoods 
    are smoothed less. This not only overcomes data sparsity but also incorporates 
    spatial context into the gene expression data. The resulting smoothed gene 
    expression data is used for clustering and could be used for other downstream 
    analyses.
License: GPL-2
Encoding: UTF-8
URL: https://sydneybiox.github.io/clustSIGNAL/
BugReports: https://github.com/sydneybiox/clustSIGNAL/issues
biocViews: Clustering, Software, GeneExpression, Spatial,
        Transcriptomics, SingleCell
RoxygenNote: 7.3.2
Depends: R (>= 4.4.0), SpatialExperiment
Imports: BiocParallel, BiocNeighbors, bluster (>= 1.16.0), scater,
        harmony, SingleCellExperiment, SummarizedExperiment, methods,
        Matrix, reshape2
Suggests: knitr, BiocStyle, testthat (>= 3.0.0), aricode, ggplot2,
        patchwork, dplyr, scattermore
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/clustSIGNAL
git_branch: RELEASE_3_23
git_last_commit: 420ae66
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:18:47 UTC; biocbuild
Author: Pratibha Panwar [cre, aut, ctb] (ORCID:
    <https://orcid.org/0000-0002-7437-7084>),
  Boyi Guo [aut],
  Haowen Zhao [aut],
  Stephanie Hicks [aut],
  Shila Ghazanfar [aut, ctb] (ORCID:
    <https://orcid.org/0000-0001-7861-6997>)
Maintainer: Pratibha Panwar <pratibhapanwar.4@gmail.com>

# Generated by roxygen2: do not edit by hand

export(adaptiveSmoothing)
export(clustSIGNAL)
export(entropyMeasure)
export(neighbourDetect)
export(p1_clustering)
export(p2_clustering)
importFrom(BiocNeighbors,KmknnParam)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(Matrix,sparseMatrix)
importFrom(SingleCellExperiment,logcounts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(bluster,KmeansParam)
importFrom(bluster,NNGraphParam)
importFrom(bluster,TwoStepParam)
importFrom(bluster,clusterRows)
importFrom(harmony,RunHarmony)
importFrom(methods,as)
importFrom(methods,show)
importFrom(reshape2,melt)
importFrom(scater,runPCA)
importFrom(stats,dexp)
importFrom(stats,dnorm)
importFrom(stats,setNames)

# ClustSIGNAL

<img src="man/figures/clustSIGNAL_hex.jpg" align="right" height="250" width="225"/>

ClustSIGNAL: ***Clust***ering of ***S***patially ***I***nformed ***G***ene expression with ***N***eighbourhood ***A***dapted ***L***earning.

An R package to perform spatially-informed cell type clustering on high-resolution spatial transcriptomics data. This method has two-fold motivation: overcoming data sparsity by using neighbourhood heterogeneity information to guide an adaptive smoothing approach, and performing spatially-informed clustering by embedding spatial context into the gene expression. To achieve these, we calculate entropy as a measure of heterogeneity of cell neighbourhoods and use it to generate weight distributions and perform adaptive smoothing of gene expression. Homogeneous neighbourhoods generally have low entropy, and so, smoothing is performed over more cells in these neighbourhoods. Contrarily, heterogeneous neighbourhoods have high entropy and are smoothed over a much smaller region. The resulting adaptively smoothed gene expression is used for clustering.

For a tutorial on how to use ClustSIGNAL, see the vignette at this [website](https://sydneybiox.github.io/clustSIGNAL/).

## Installation

To install ClustSIGNAL via Bioconductor:

``` r
library(BiocManager)
BiocManager::install("clustSIGNAL")
```

To install ClustSIGNAL from [GitHub](https://github.com/SydneyBioX/clustSIGNAL):

``` r
# install.packages("devtools")
devtools::install_github("SydneyBioX/clustSIGNAL")
```

## Method description

<img src="man/figures/Fig1_schematic.jpg" align="middle"/>

**Figure: ClustSIGNAL method overview.**

Here, we present ClustSIGNAL, a spatial clustering method developed to handle data sparsity while considering the variability in cell arrangement of tissue regions. The core steps involved in the method are sequential:

**1.** The method starts with non-spatial clustering and subclustering (default louvain clustering) to classify cells into clusters and subclusters that we refer to as ‘initial clusters’ and ‘initial subclusters’, respectively.

**2.** The neighbourhood of each cell is then defined in terms of initial subcluster composition.

**3.** The cells in the neighbourhood are also sorted and rearranged so that the neighbours belonging to the same initial clusters as the index cell are placed closer to it.

**4.** Neighbourhood heterogeneity is measured as entropy, where a high entropy value indicates more heterogeneity in the neighbourhood and a low entropy value indicates a more homogeneous neighbourhood.

**5.** The entropy values are used to generate weight distributions specific to each neighbourhood.

**6.** The gene expressions of cells are adaptively smoothed using the entropy-guided weight distributions; cells in heterogeneous neighbourhoods (high entropy regions) undergo smoothing over a smaller region, whereas cells in homogeneous neighbourhoods (low entropy regions) undergo smoothing over a larger region.

**7.** Non-spatial clustering is performed with adaptively smoothed gene expression to generate ClustSIGNAL clusters.

## ClustSIGNAL parameters

ClustSIGNAL package uses a SpatialExperiment object as input. We provide users with a number of parameters to explore and experiment with, as well as prior tested default values for quick runs. ClustSIGNAL can be used for single sample or multisample analysis with just one function call. Below is the list of the parameters offered and their possible values:

-   **spe** - SpatialExperiment object with cell spatial coordinates and normalized gene expression in it.

-   **samples** - column name in cell metadata of the spe object that contains sample names.

-   **dimRed** - dimensionality reduction method name in reduced dimensions of the spe object. Default value is "None", in which case PCA is calculated and used as low dimension data.

-   **batch** - whether batch correction should be performed. Default value is False.

-   **batch_by** - column name in cell metadata of the spe object containing the groups by which to perform batch correction. By default, batch_by is set to "None". If the batch option is set to True, the batch_by value must be changed to the group name to perform batch correction.

-   **NN** - neighbourhood size in terms of the number of nearest neighbours to consider. Value should be \> 1. Default value is 30.

-   **kernel** - type of weight distribution to use. Can be Gaussian (default) or exponential distribution.

-   **spread** - value of distribution parameter - standard deviation of Gaussian distribution or rate of exponential distribution. Default value is 0.3, recommended for Gaussian distribution. For exponential distribution, recommended value is 5.

-   **sort** - whether cell neighbourhoods should be sorted by their initial cluster labels. Default value is True.

-   **threads** - number of cpus to use for parallel runs. Default value is 1.

-   **outputs** - choice of output types. Default value is 'c' for a list containing data frame of cell IDs and ClustSIGNAL cluster labels. Other possible values are "n" for a list of ClustSIGNAL clusters dataframe and neighbourhood matrix, "s" for a list of ClustSIGNAL clusters dataframe and final SpatialExperiment object, and "a" for a list of all 3 outputs - ClustSIGNAL clusters dataframe, neighbourhood matrix, and final spe object.

-   **clustParams** - parameter options for TwoStepParam clustering methods in the bluster package. The clustering parameters are in the following order in this list:

    -   ***clust_c*** represents number of centers for clustering with KmeansParam. By default this is set to 0, in which case the method uses either 5000 centers or 1/5th of the total cells in the data as the number of centers, whichever is lower.

    -   ***subclust_c*** represents number of centers for sub-clustering the initial clusters with KmeansParam. The default value is 0, in which case the method uses either 1 center or half of the total cells in the initial cluster as the number of centers, whichever is higher.

    -   ***iter.max*** for the maximum number of iterations to perform during clustering and subclustering with KmeansParam. Default value is 30.

    -   ***k*** indicates the k-value used for clustering/subclustering with NNGraphParam. Default value is 10.

    -   ***cluster.fun*** is the graph clustering method used with NNGraphParam. By default, the Louvain method is used.

## Running ClustSIGNAL

Before running ClustSIGNAL, it is important to ensure that the input SpatialExperiment object has spatial coordinates stored in the spatialCoords matrix. Otherwise, the method will throw an error asking the user to provide spatial coordinates.

``` r
# load required packages
library(clustSIGNAL)

data(ClustSignal_example)

# Here, the sample labels are in 'sample_id' column.
res <- clustSIGNAL(spe, samples = "sample_id", outputs = "a")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = FALSE, warning = FALSE)

## ----load_packages, message = FALSE, warning = FALSE--------------------------
# load required packages
library(clustSIGNAL)
library(scater)
library(ggplot2)
library(dplyr)
library(patchwork)
library(aricode)

## ----embryo_data_prep---------------------------------------------------------
# load me_expr containing gene expression logcounts
# load me_data containing cell metadata including x-y coordinates
data(mEmbryo2)
# to create a SpatialExperiment object we need gene expression, cell metadata, 
# and cell locations.
spe <- SpatialExperiment::SpatialExperiment(
  assays = list(logcounts = me_expr), 
  colData = me_data,
  # spatialCoordsNames requires column names in me_data that contain 
  # xy-coordinates of cells
  spatialCoordsNames = c("X", "Y"))
spe

## ----embryo_data_columns------------------------------------------------------
spe |> colData() |> colnames() # column names in the metadata

## ----ClustSIGNAL_singleRun----------------------------------------------------
set.seed(100)
samples <- "sample_id" # column name containing sample names
# to run ClustSIGNAL, requires a SpatialExperiment object, column name of sample
# labels in colData slot, and the output type to generate (clusters, neighbours,
# and/or final spe object).
res_emb <- clustSIGNAL(spe, samples, outputs = "a") 

## ----embryo_result_list-------------------------------------------------------
res_emb |> names() # names of the outputs generated

## ----embryo_clusters_head-----------------------------------------------------
res_emb$clusters |> head() # cluster data frame has cell IDs and cluster labels

## ----embryo_final_spe---------------------------------------------------------
# for convenience with downstream analyses, we will replace the original spe
# object with the one generated by ClustSIGNAL. This does not lead to any loss 
# of information as ClustSIGNAL only adds information to the input spe object.
spe <- res_emb$spe_final
spe
spe |> colData() |> colnames()

## ----colors-------------------------------------------------------------------
colors <- c("#635547", "#8EC792", "#9e6762", "#FACB12", "#3F84AA", "#0F4A9C", 
            "#ff891c", "#EF5A9D", "#C594BF", "#DFCDE4", "#139992", "#65A83E", 
            "#8DB5CE", "#005579", "#C9EBFB", "#B51D8D", "#532C8A", "#8870ad", 
            "#cc7818", "#FBBE92", "#EF4E22", "#f9decf", "#c9a997", "#C72228", 
            "#f79083", "#F397C0", "#DABE99", "#c19f70", "#354E23", "#C3C388",
            "#647a4f", "#CDE088", "#f7f79e", "#F6BFCB", "#7F6874", "#989898", 
            "#1A1A1A", "#FFFFFF", "#e6e6e6", "#77441B", "#F90026", "#A10037", 
            "#DA5921", "#E1C239", "#9DD84A")

## ----embryo_spatialPlots1-----------------------------------------------------
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension slot of the spe object
reducedDim(spe, "spatial") <- spatialCoords(spe)

## ----embryo_spatialPlots2-----------------------------------------------------
# spatial plot
spt_clust <- scater::plotReducedDim(
  spe, colour_by = "ClustSIGNAL", dimred = "spatial", point_alpha = 1,
  point_size = 4, scattermore = TRUE) +
  ggtitle("A. Spatial plot of clusters") +
  scale_color_manual(values = colors) +
  guides(colour = guide_legend(title = "Clusters", 
                               override.aes = list(size = 5))) +
  theme(text = element_text(size = 12))

## ----embryo_spatialPlots3-----------------------------------------------------
# entropy distribution plotted at cluster-level can indicate which clusters 
# have cells from homogeneous/heterogeneous space. 
df_met <- spe |> colData() %>% as.data.frame()
ct_ent <- df_met %>% 
  mutate(ClustSIGNAL = as.character(ClustSIGNAL)) %>%
  group_by(ClustSIGNAL) %>%
  # calculating median entropy of each cluster category
  summarise(mdEntropy = median(entropy)) %>% 
  # reordering clusters by their median entropy value
  arrange(mdEntropy)
df_met$ClustSIGNAL <- factor(df_met$ClustSIGNAL, levels = ct_ent$ClustSIGNAL)
col_ent <- colors[as.numeric(as.character(ct_ent$ClustSIGNAL))]
box_clust <- df_met %>%
  ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
  geom_boxplot() +
  scale_fill_manual(values = col_ent) +
  ggtitle("B. Entropy distribution of clusters") +
  labs(x = "ClustSIGNAL clusters", y = "Entropy", name = "Clusters") +
  theme_classic() +
  theme(legend.position = "none",
        text = element_text(size = 12),
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1),
        plot.title = element_text(face = "bold"))

## ----embryo_spatialPlots4-----------------------------------------------------
spt_clust + box_clust + patchwork::plot_layout(guides = "collect", 
                                               widths = c(2, 3))

## ----embryo_clusterMetrics----------------------------------------------------
# to assess the accuracy of clustering, the cluster labels are often compared to
# prior annotations. Here, we compare ClustSIGNAL cluster labels to annotations 
# available with this public data.
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(
    ARI = aricode::ARI(celltype_mapped_refined, ClustSIGNAL), # calculate ARI
    NMI = aricode::NMI(celltype_mapped_refined, ClustSIGNAL)) # calculate NMI

## ----embryo_entropyMetrics----------------------------------------------------
# we can assess the overall entropy distribution of the dataset
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(min_Entropy = min(entropy),
            min_Entropy_count = sum(spe$entropy == 0),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))

## ----entropyPlots1------------------------------------------------------------
# we can also visualize the distribution and spread of the entropy values
hst_ent <- spe |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  ggtitle("A. Entropy spread") +
  labs(x = "Entropy", y = "Number of neighbourhoods") +
  theme_classic() +
  theme(text = element_text(size = 12),
        plot.title = element_text(face = "bold"))

## ----entropyPlots2------------------------------------------------------------
spt_ent <- scater::plotReducedDim(spe, colour_by = "entropy",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  ggtitle("B. Entropy spatial distribution") +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe$entropy))) +
  theme(text = element_text(size = 12))

## ----entropyPlots3------------------------------------------------------------
hst_ent + spt_ent

## ----hypothal_data_prep-------------------------------------------------------
# load mh_expr containing gene expression logcounts
# load mh_data containing cell metadata and cell x-y coordinates
data(mHypothal)
# create spe object using gene expression, cell metadata, and cell locations
spe2 <- SpatialExperiment(assays = list(logcounts = mh_expr), 
                          colData = mh_data,
                          # spatialCoordsNames requires column names in 
                          # mh_data that contain xy-coordinates of cells
                          spatialCoordsNames = c("X", "Y"))
spe2

## ----hypothal_data_columns----------------------------------------------------
spe2 |> colData() |> str() # metadata summary

## ----ClustSIGNAL_multiRun-----------------------------------------------------
set.seed(110)
# ClustSIGNAL can be run on a dataset with multiple samples. As before, we need
# the SpatialExperiment object and column name of sample labels in the object. 
# The method can be run in parallel through the threads option. Here we use 
# thread = 4 to use 4 cores.
# Since no batch effects were observed in this data subset, we have not used 
# the batch and batch_by options.
samples <- "samples" # column name containing sample names
res_hyp <- clustSIGNAL(spe2, samples, threads = 4, outputs = "a")

## ----hypothal_final_spe-------------------------------------------------------
# for convenience with downstream analyses, we replace the original spe object 
# with the one generated by ClustSIGNAL.
spe2 <- res_hyp$spe_final
spe2

## ----hypothal_samples---------------------------------------------------------
samplesList <- spe2[[samples]] |> levels() # get sample names
samplesList

## ----hypothal_clusterMetrics--------------------------------------------------
spe2 |> colData() %>% 
  as.data.frame() %>%
  group_by(samples) %>%
  summarise(
    # Comparing ClustSIGNAL cluster labels to annotations available with the 
    # public data to assess its accuracy.
    ARI = aricode::ARI(Cell_class, ClustSIGNAL),
    NMI = aricode::NMI(Cell_class, ClustSIGNAL),
    # Assessing the overall entropy distribution of the samples in the dataset.
    min_Entropy = min(entropy),
    min_Entropy_count = sum(entropy == 0),
    max_Entropy = max(entropy),
    mean_Entropy = mean(entropy))

## ----hypothal_spatialPlots1---------------------------------------------------
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension section
reducedDim(spe2, "spatial") <- spatialCoords(spe2)

## ----hypothal_spatialPlots2---------------------------------------------------
# spatial plot - ClustSIGNAL clusters
spt_clust2 <- scater::plotReducedDim(spe2, colour_by = "ClustSIGNAL",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  scale_color_manual(values = colors) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  guides(colour = guide_legend(title = "Clusters",
                               override.aes = list(size = 3))) +
  theme(text = element_text(size = 12))

## ----hypothal_spatialPlots3---------------------------------------------------
# For visualising cluster-level entropy distribution, we reorder the clusters 
# by their median entropy value in each sample
df_met2 <- spe2 |> colData() %>% as.data.frame()

box_clust2 <- list()
for (s in samplesList) {
  df_met_sub <- df_met2[df_met2[[samples]] == s, ]
  # calculating median entropy of each cluster in a sample
  ct_ent2 <- df_met_sub %>%
    mutate(ClustSIGNAL = as.character(ClustSIGNAL)) %>%
    group_by(ClustSIGNAL) %>%
    summarise(mdEntropy = median(entropy)) %>%
    # reordering clusters by their median entropy
    arrange(mdEntropy)
  
  df_met_sub$ClustSIGNAL <- factor(df_met_sub$ClustSIGNAL, 
                                   levels = ct_ent2$ClustSIGNAL)
  # box plot of cluster entropy
  col_ent2 <- colors[as.numeric(ct_ent2$ClustSIGNAL)]
  box_clust2[[s]] <- df_met_sub %>%
    ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
    geom_boxplot() +
    scale_fill_manual(values = col_ent2) +
    facet_wrap(vars(samples), nrow = 1) +
    labs(x = "ClustSIGNAL clusters", y = "Entropy") +
    ylim(0, NA) +
    theme_classic() +
    theme(strip.text = element_blank(),
          legend.position = "none",
          text = element_text(size = 12),
          axis.text.x = element_text(angle = 90, vjust = 0.5))
}

## ----hypothal_spatialPlots4---------------------------------------------------
spt_clust2 / (patchwork::wrap_plots(box_clust2[1:3], nrow = 1) +
                plot_layout(axes = "collect")) +
  plot_layout(guides = "collect", heights = c(5, 3)) +
  plot_annotation(
    title = "Spatial (top) and entropy (bottom) distributions of clusters",
    theme = theme(plot.title = element_text(hjust = 0.5, face = "bold")))

## ----hypothal_entropyPlots1---------------------------------------------------
hst_ent2 <- spe2 |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  facet_wrap(vars(samples), nrow = 1) +
  labs(x = "Entropy", y = "Number of neighbourhoods") +
  theme_classic() +
  theme(text = element_text(size = 12))

## ----hypothal_entropyPlots2---------------------------------------------------
spt_ent2 <- scater::plotReducedDim(spe2, colour_by = "entropy",
                                  # specify spatial low dimension
                                  dimred = "spatial", point_alpha = 1,
                                  point_size = 4, scattermore = TRUE) +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe2$entropy))) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  theme(strip.text = element_blank(),
        text = element_text(size = 12))

## ----hypothal_entropyPlots3---------------------------------------------------
hst_ent2 / spt_ent2 + plot_layout(heights = c(4, 5)) +
    plot_annotation(
      title = "Entropy spread (top) and spatial distribution (bottom)",
      theme = theme(plot.title = element_text(hjust = 0.5, face = "bold")))

## ----ClustSIGNALseq_data------------------------------------------------------
# load logcounts and metadata to the environment
data(mEmbryo2)

# as before, we read the data into a SpatialExperiment object
spe <- SpatialExperiment(assays = list(logcounts = me_expr),
                         colData = me_data, spatialCoordsNames = c("X", "Y"))

## ----ClustSIGNALseq_prep------------------------------------------------------
set.seed(100)
# first we need to generate low dimension data for initial clustering
spe <- scater::runPCA(spe) 

## ----ClustSIGNALseq_step1-----------------------------------------------------
spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA")

## ----ClustSIGNALseq_step1_out1------------------------------------------------
spe$initCluster |> head() # clustering output

## ----ClustSIGNALseq_step1_out2------------------------------------------------
spe$initSubcluster |> head() # subclustering output

## ----ClustSIGNALseq_step2-----------------------------------------------------
# This step generates a list of neighbourhood information.
outReg <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id")

## ----ClustSIGNALseq_step2_out1------------------------------------------------
outReg$nnCells[1:3, 1:3]

## ----ClustSIGNALseq_step2_out2------------------------------------------------
outReg$regXclust[[1]] 

## ----ClustSIGNALseq_step3-----------------------------------------------------
spe <- clustSIGNAL::entropyMeasure(spe, outReg$regXclust)

## ----ClustSIGNALseq_step3_out-------------------------------------------------
spe$entropy |> head() # entropy values

## ----ClustSIGNALseq_step4-----------------------------------------------------
spe <- clustSIGNAL::adaptiveSmoothing(spe, outReg$nnCells)

## ----ClustSIGNALseq_step4_out-------------------------------------------------
assay(spe, "smoothed")[1:5, 1:3]

## ----ClustSIGNALseq_step5-----------------------------------------------------
spe <- clustSIGNAL::p2_clustering(spe)

## ----ClustSIGNALseq_step5_out-------------------------------------------------
spe$ClustSIGNAL |> head() # ClustSIGNAL cluster labels

## -----------------------------------------------------------------------------
sessionInfo()

#' Adaptive smoothing
#'
#' @description
#' A function to perform a weighted, adaptive smoothing of the gene expression
#' of each cell based on the heterogeneity of its neighbourhood. Heterogeneous
#' neighbourhoods are smoothed less with higher weights given to cells belonging
#' to same initial cluster as the index cell. Homogeneous neighbourhoods are
#' smoothed more with similar weights given to most cells.
#' @param spe SpatialExperiment object containing neighbourhood entropy values
#' of each cell.
#' @param nnCells a character matrix of NN nearest neighbours - rows are index
#' cells and columns are their nearest neighbours ranging from closest to
#' farthest neighbour. For sort = TRUE, the neighbours belonging to the same
#' initial cluster as the index cell are moved closer to it.
#' @param NN an integer for the number of neighbouring cells the function
#' should consider. The value must be greater than or equal to 1. Default value
#' is 30.
#' @param kernel a character for type of distribution to be used. The two valid
#' values are "G" or "E" for Gaussian and exponential distributions,
#' respectively. Default value is "G".
#' @param spread a numeric value for distribution spread, represented by
#' standard deviation for Gaussian distribution and rate for exponential
#' distribution. Default value is 0.3 for Gaussian distribution. The recommended
#' value is 5 for exponential distribution.
#' @return SpatialExperiment object including smoothed gene expression as an
#' additional assay.
#' @importFrom SingleCellExperiment logcounts
#' @importFrom SummarizedExperiment assay
#' @importFrom methods show as
#' @importFrom BiocParallel bplapply
#' @importFrom Matrix sparseMatrix
#' @importFrom reshape2 melt
#' @examples
#' data(ClustSignal_example)
#'
#' # requires matrix containing NN nearest neighbour cell labels (nnCells),
#' # generated using the neighbourDetect() function
#' spe <- clustSIGNAL::adaptiveSmoothing(spe, nnCells)
#' spe
#' @export

adaptiveSmoothing <- function(spe, nnCells, NN = 30, kernel = "G",
                              spread = 0.3) {
    G_mat <- as(logcounts(spe), "sparseMatrix")
    if (kernel == "G") { # generate normal distribution weights
        wts <- .gauss_kernel(spe$entropy, NN + 1, spread)
    } else if (kernel == "E") { # generate exponential distribution weights
        wts <- .exp_kernel(spe$entropy, NN + 1, spread)}
    colnames(wts) <- colnames(spe)
    wts <- sweep(wts, 2, colSums(wts), FUN = "/") # scale weights
    wts_df <- cbind(reshape2::melt(t(nnCells))[, 2:3], reshape2::melt(wts)[, 3])
    colnames(wts_df) <- c("c", "ci", "wci")
    wts_df$c <- as.integer(match(wts_df$c, colnames(spe)))
    wts_df$ci <- as.integer(match(wts_df$ci, colnames(spe)))
    W_mat <- Matrix::sparseMatrix(i = wts_df$ci, j = wts_df$c,
                                  x = wts_df$wci)
    smoothMat <- G_mat %*% W_mat
    colnames(smoothMat) <- colnames(spe)
    # QC check
    check.genes <- identical(rownames(spe), rownames(smoothMat))
    check.NA <- sum(is.na(smoothMat))
    if (check.genes == FALSE) {
        stop("Gene order in smoothed data does not match gene order
             in SpatialExperiment object.")
    } else if (check.NA != 0) {
        stop("Smoothed data has missing values.")
    } else {
        SummarizedExperiment::assay(spe, "smoothed") <- smoothMat
        show(paste("Smoothing performed. NN =", NN, "Kernel =", kernel,
                   "Spread =", spread, "Time", format(Sys.time(),'%H:%M:%S')))}
    return(spe)
}

#' Final non-spatial clustering
#'
#' @description
#' A function to perform clustering on adaptively smoothed gene expression data
#' to generate ClustSIGNAL clusters.
#' @param spe SpatialExperiment object containing the adaptively smoothed gene
#' expression.
#' @param batch a logical parameter for whether to perform batch correction.
#' Default value is FALSE.
#' @param batch_by a character indicating name of colData(spe) column containing
#' the batch names. Default value is 'None'.
#' @param threads a numeric value for the number of CPU cores to be used for the
#' analysis. Default value set to 1.
#' @param clustParams  a list of parameters for TwoStepParam clustering methods:
#' clust_c is the number of centers to use for clustering with KmeansParam. By
#' default set to 0, in which case the method uses either 5000 centers or 1/5th
#' of the total cells in the data as the number of centers, whichever is lower.
#' subclust_c is the number of centers to use for sub-clustering the initial
#' clusters with KmeansParam. This parameter is not used in the final
#' clustering step.
#' iter.max is the maximum number of iterations to perform during clustering
#' and sub-clustering with KmeansParam. Default value is 30.
#' k is a numeric value indicating the k-value used for clustering with
#' NNGraphParam. Default value is 10.
#' cluster.fun is a character indicating the graph clustering method used with
#' NNGraphParam. By default, the Louvain method is used.
#' @return SpatialExperiment object containing clusters generated from smoothed
#' data.
#' @importFrom bluster clusterRows TwoStepParam KmeansParam NNGraphParam
#' @importFrom scater runPCA
#' @importFrom harmony RunHarmony
#' @importFrom SummarizedExperiment colData
#' @importFrom SingleCellExperiment reducedDim
#' @importFrom methods show
#' @importFrom stats setNames
#' @examples
#' data(ClustSignal_example)
#'
#' # For non-spatial clustering of normalised counts
#' spe <- clustSIGNAL::p2_clustering(spe)
#' spe$ClustSIGNAL |> head()
#' @export

#### Non-spatial clustering
p2_clustering <- function(spe, batch = FALSE, batch_by = "None", threads = 1,
                          clustParams = list(clust_c = 0, subclust_c = 0,
                                             iter.max = 30, k = 10,
                                             cluster.fun = "louvain")) {
    if (clustParams[[1]] == 0)
        # number of centers = 1/5th of total cells in sample
        clustVal <- min(as.integer(ncol(spe) / 5), 5000) else
            clustVal <- clustParams[[1]]
    # Clustering adaptively smoothed data
    spe <- scater::runPCA(spe, assay.type = "smoothed", name = "PCA.smooth")
    if (batch == TRUE) {
        emb <- harmony::RunHarmony(data_mat = reducedDim(spe, "PCA.smooth"),
                                   meta_data = colData(spe),
                                   vars_use = batch_by, max.iter = 20,
                                   verbose = FALSE)
        mat <- emb
    } else {
        mat <- reducedDim(spe, "PCA.smooth")}
    reClust <- bluster::clusterRows(
        mat,
        bluster::TwoStepParam(
            first = bluster::KmeansParam(centers = clustVal,
                                         iter.max = clustParams[[3]]),
            second = bluster::NNGraphParam(k = clustParams[[4]],
                                           cluster.fun = clustParams[[5]],
                                           num.threads = threads)))
    spe$ClustSIGNAL <- factor(reClust)
    show(paste("Nonspatial clustering performed on smoothed data. Clusters =",
               length(unique(reClust)), "Time",
               format(Sys.time(),'%H:%M:%S')))
    return (spe)
}

#' Heterogeneity measure
#'
#' @description
#' A function to measure the heterogeneity of a cell's neighbourhood in terms of
#' entropy. Generally, homogeneous neighbourhoods have low entropy and
#' heterogeneous neighbourhoods have high entropy.
#' @param spe SpatialExperiment object with initial cluster and subcluster
#' labels.
#' @param regXclust a list of vectors of each cell's neighbourhood composition
#' indicated by the proportion of initial subclusters it contains.
#' @param threads a numeric value for the number of CPU cores to be used for the
#' analysis. Default value set to 1.
#' @return SpatialExperiment object with entropy values associated with each
#' cell.
#' @importFrom BiocParallel bplapply
#' @importFrom methods show
#' @examples
#' data(ClustSignal_example)
#'
#' # requires list containing cluster proportions of each region (regXclust),
#' # generated using the neighbourDetect() function
#' spe <- clustSIGNAL::entropyMeasure(spe, regXclust)
#' spe$entropy |> head()
#' @export

entropyMeasure <- function(spe, regXclust, threads = 1) {
    cell_vect <- as.vector(colnames(spe))
    BPPARAM <- .generateBPParam(cores = threads)
    regEntropy <- BiocParallel::bplapply(cell_vect, function(c){
        arr <- as.vector(unlist(regXclust[c]))
        y <- round(matrix(-sum(arr * log2(arr)), nrow = 1, ncol = 1), 5)
        rownames(y) <- c
        y}, BPPARAM = BPPARAM)
    # QC check
    check.cells <- identical(sapply(regEntropy, rownames), colnames(spe))
    check.NA.values <- sum(is.na(unlist(regEntropy)))
    if (check.cells == FALSE) {
        stop("Cell order in entropy data does not match cell order in Spatial
             Experiment object")
    } else if (check.NA.values != 0) {
        stop("Entropy values are missing for some cells.")
    } else {
        spe$entropy <- as.vector(unlist(regEntropy))
        show(paste("Region heterogeneity calculated. Time",
                   format(Sys.time(),'%H:%M:%S')))}
    return(spe)
}

#' Generating BPParam object
#'
#' @description
#' A utility function to generate BPPARAM object.
#' @param cores Desired number of cores for BPPARAM object.
#' @importFrom BiocParallel SerialParam SnowParam MulticoreParam bpparam
#' @return A BPPPARAM object.
#' @keywords internal

.generateBPParam <- function(cores = 1) {
    seed <- .Random.seed[1]
    if (cores == 1) {
        BPparam <- BiocParallel::SerialParam(RNGseed = seed)
    } else { # Parallel processing is desired.
        # Set BPparam RNGseed if the user ran set.seed(someNumber) themselves
        if (Sys.info()["sysname"] == "Windows") {
            # Only SnowParam suits Windows
            BPparam <- BiocParallel::SnowParam(min(
                cores,
                BiocParallel::snowWorkers("SOCK")
            ),
            RNGseed = seed
            )
        } else if (Sys.info()["sysname"] %in% c("MacOS", "Linux")) {
            BPparam <- BiocParallel::MulticoreParam(min(
                cores,
                BiocParallel::multicoreWorkers()
            ),
            RNGseed = seed
            )
            # Multicore is faster than SNOW, but it does not work on Windows
        } else {
            BPparam <- BiocParallel::bpparam()}}
    BPparam
}


#' Neighbour cell sorting
#'
#' @description
#' A function to perform neighbourhood cell sorting. Neighbourhood cells that
#' belong to the same initial cluster as the index cell are moved closer to the
#' index cell.
#' @param cell a vector of neighbourhood cell indices. The cell indices indicate
#' the row number of cells in sample metadata.
#' @param Clust a data frame of initial cluster labels of each cell in the
#' sample.
#' @return a data frame of cell IDs of sorted neighbourhood cells.
#' @keywords internal

.cellNameSort <- function (cell, Clust) {
    cellDF <- data.frame(cellName = rownames(Clust)[cell],
                         cluster = Clust[cell, 1])
    if (length(unique(Clust[cell,])) == 1) { # cells from same initial cluster
        return(rownames(Clust)[cell])
    } else if (cellDF$cluster[1] %in% cellDF$cluster[seq(2, length(cell))]) {
        centralClust <- cellDF$cluster[1]
        sameClust <- c()
        diffClust <- c()
        for (c in seq_len(nrow(cellDF))) {
            if (cellDF$cluster[c] == centralClust){
                sameClust <- append(sameClust, cellDF$cellName[c])
            } else {
                diffClust <- append(diffClust, cellDF$cellName[c])}}
        region <- c(sameClust, diffClust)
        return(region)
    } else {
        return(rownames(Clust)[cell])}
}


#' Neighbour cell naming
#'
#' @description
#' A function to fetch cell IDs.
#' @param cell a vector of neighbourhood cell indices. The cell indices indicate
#' the row number of cells in sample metadata.
#' @param Clust a data frame of initial cluster labels of each cell in the
#' sample.
#' @return a data frame of cell IDs of neighbourhood cells.
#' @keywords internal

.cellName <- function (cell, Clust) {
    return(rownames(Clust)[cell])
}


#' Neighbour cell initial subcluster label
#'
#' @description
#' A function to fetch initial subcluster label of cell.
#' @param cell a vector of neighbourhood cell indices. The cell indices indicate
#' the row number of cells in sample metadata.
#' @param subClust a data frame of initial subcluster labels of each cell in
#' the sample.
#' @return a data frame of initial subcluster labels of neighbourhood
#' cells.
#' @keywords internal

.clustNum <- function (cell, subClust) {
    return(subClust[cell,])
}


#' Cell neighbourhood composition
#'
#' @description
#' A function to calculate the cell neighbourhood composition of initial cluster
#' labels.
#' @param arr a vector of initial subcluster labels of each cell in the
#' neighbourhood.
#' @return a table of initial subcluster proportions in a neighbourhood.
#' @keywords internal

.calculateProp <- function(arr) {
    prop <- prop.table(table(arr))
    # QC check
    if (round(sum(prop)) != 1) {
        stop("Proportions are incorrect.", "row =", c, "proportion =", sum(arr))
    }
    return(prop)
}


#' Gaussian distribution weights
#'
#' @description
#' A function to generate weights from a Gaussian distribution.
#' @param ed a numeric vector of entropy values of all cell neighbourhoods.
#' @param NN an integer for the number of neighbourhood cells including the
#' index cell.
#' @param sd a numeric value for standard deviation of Gaussian distribution.
#' @importFrom stats dnorm
#' @return a matrix where the columns contain weights associated with the
#' entropy values.
#' @keywords internal

.gauss_kernel <- function(ed, NN, sd) {
    weight_mat <- dnorm(sapply(ed, function(x) seq(0, x, length.out = NN)),
                        sd = sd)
    return (weight_mat)
}


#' Exponential distribution weights
#'
#' @description
#' A function to generate weights from an exponential distribution.
#' @param ed a numeric vector of entropy values of all cell neighbourhoods.
#' @param NN an integer for the number of neighbourhood cells including the
#' index cell.
#' @param rate a numeric value for rate of exponential distribution.
#' @importFrom stats dexp
#' @return a matrix where the columns contain weights associated with the
#' entropy values.
#' @keywords internal

.exp_kernel <- function(ed, NN, rate) {
    weight_mat <- dexp(sapply(ed, function(x) seq(0, x, length.out = NN)),
                       rate = rate)
    return (weight_mat)
}

#' ClustSIGNAL
#'
#' @description
#' A clustering method for spatially-resolved cell-type classification of
#' spatial transcriptomics data. The tool generates and uses an adaptively
#' smoothed, spatially informed gene expression data for clustering.
#' @param spe a SpatialExperiment object containing spatial coordinates in
#' 'spatialCoords' matrix and normalised gene expression in 'logcounts' assay.
#' @param samples a character indicating name of colData(spe) column containing
#' sample names.
#' @param dimRed a character indicating the name of the reduced dimensions to
#' use from the SpatialExperiment object (i.e., from reducedDimNames(spe)).
#' Default value is 'None'.
#' @param batch a logical parameter for whether to perform batch correction.
#' Default value is FALSE.
#' @param batch_by a character indicating name of colData(spe) column containing
#' the batch names. Default value is 'None'.
#' @param NN an integer for the number of neighbouring cells the function
#' should consider. The value must be greater than or equal to 1. Default value
#' is 30.
#' @param kernel a character for type of distribution to be used. The two valid
#' values are "G" or "E" for Gaussian and exponential distributions,
#' respectively. Default value is "G".
#' @param spread a numeric value for distribution spread, represented by
#' standard deviation for Gaussian distribution and rate for exponential
#' distribution. Default value is 0.3 for Gaussian distribution. The recommended
#' value is 5 for exponential distribution.
#' @param sort a logical parameter for whether to sort the neighbourhood by
#' initial clusters. Default value is TRUE.
#' @param threads a numeric value for the number of CPU cores to be used for the
#' analysis. Default value set to 1.
#' @param clustParams a list of parameters for TwoStepParam clustering methods:
#' clust_c is the number of centers to use for clustering with KmeansParam. By
#' default set to 0, in which case the method uses either 5000 centers or 1/5th
#' of the total cells in the data as the number of centers, whichever is lower.
#' subclust_c is the number of centers to use for sub-clustering the initial
#' clusters with KmeansParam. The default value is 0, in which case the method
#' uses either 1 center or half of the total cells in the initial cluster as
#' the number of centers, whichever is higher.
#' iter.max is the maximum number of iterations to perform during clustering
#' and sub-clustering with KmeansParam. Default value is 30.
#' k is a numeric value indicating the k-value used for clustering and
#' sub-clustering with NNGraphParam. Default value is 10.
#' cluster.fun is a character indicating the graph clustering method used with
#' NNGraphParam. By default, the Louvain method is used.
#' @param outputs a character for the type of output to return to the user. "c"
#' for data frame of cell IDs and their respective ClustSIGNAL cluster labels,
#' "n" for ClustSIGNAL cluster dataframe plus neighbourhood matrix, "s" for
#' ClustSIGNAL cluster dataframe plus final SpatialExperiment object, or "a" for
#' all 3 outputs.
#' @return a list of outputs depending on the type of outputs specified in the
#' main function call.
#'
#' 1. clusters: a data frame of cell names and their ClustSIGNAL cluster
#' classification.
#'
#' 2. neighbours: a character matrix containing cells IDs of each cell's
#' NN neighbours.
#'
#' 3. spe_final: a SpatialExperiment object containing the original spe object
#' data plus initial cluster and subcluster labels, entropy values, smoothed
#' gene expression, and ClustSIGNAL cluster labels.
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom SingleCellExperiment reducedDimNames
#' @importFrom methods show
#' @importFrom scater runPCA
#' @examples
#' data(ClustSignal_example)
#'
#' names(colData(spe))
#' # identify the column name with sample labels
#' samples = "sample_id"
#' res_list <- clustSIGNAL(spe, samples, outputs = "c")
#' @export

clustSIGNAL <- function (spe, samples, dimRed = "None", batch = FALSE,
                         batch_by = "None", NN = 30, kernel = "G",
                         spread = 0.3, sort = TRUE, threads = 1, outputs = "c",
                         clustParams = list(clust_c = 0, subclust_c = 0,
                                            iter.max = 30, k = 10,
                                            cluster.fun = "louvain")) {
    time_start <- Sys.time()
    # input data and parameter checks
    if (length(spatialCoords(spe)) == 0) # check spatial coordinates
        stop("Spatial coordinates not found.")
    if (is.null(logcounts(spe)) == TRUE) # check normalised gene expression
        stop("Normalised gene expression not found.")
    if (is.null(spe[[samples]]) == TRUE) # check sample column
        stop("The column 'samples' not found.")
    if (length(unique(colnames(spe))) != ncol(spe)) # check cell IDs
        stop("The cell names are repeated. Provide spe object with unique cell
             names.")
    if (NN < 1) # check neighbourhood size
        stop("NN cannot be less than 1.")
    if (!(kernel %in% c("G", "E"))) # check kernel name
        stop("Invalid kernel type.")
    if (!(outputs %in% c('c', 'n', 's', 'a'))) # check output type
        stop("Invalid character for output type.")
    if (batch == TRUE & batch_by == "None") # check batch correction parameters
        stop("No group name provided for batch correction.")
    if (dimRed == "None") { # check reduced dimension data
        show(paste("Calculating PCA. Time", format(Sys.time(),'%H:%M:%S')))
        spe <- scater::runPCA(spe)
        dimRed <- "PCA"
    } else if (!(dimRed %in% reducedDimNames(spe))){
        stop("Specified low dimension data not found.")}
    show(paste("ClustSIGNAL run started. Time", format(Sys.time(),'%H:%M:%S')))

    spe <- p1_clustering(spe, dimRed, batch, batch_by, threads, clustParams)
    outReg <- neighbourDetect(spe, samples, NN, sort, threads)
    spe <- entropyMeasure(spe, outReg$regXclust, threads)
    spe <- adaptiveSmoothing(spe, outReg$nnCells, NN, kernel, spread)
    spe <- p2_clustering(spe, batch, batch_by, threads, clustParams)
    cluster_df <- data.frame("Cells" = colnames(spe),
                             "Clusters" = spe$ClustSIGNAL)
    time_end <- Sys.time()
    show(paste("ClustSIGNAL run completed.", format(Sys.time(),'%H:%M:%S')))
    show(time_end - time_start)
    if (outputs == "c"){
        return (list("clusters" = cluster_df))
    } else if (outputs == "n") {
        return (list("clusters" = cluster_df, "neighbours" = outReg$nnCells))
    } else if (outputs == "s") {
        return (list("clusters" = cluster_df, "spe_final" = spe))
    } else if (outputs == "a") {
        return (list("clusters" = cluster_df, "neighbours" = outReg$nnCells,
                     "spe_final" = spe))
    }
}

test_that(
    "Expecting error when wrong column name given for samples", {
        data(ClustSignal_example)
        samples <- "wrongID"

        testthat::expect_error(
            suppressWarnings(clustSIGNAL(spe, samples))
        )
    }
)

test_that(
    "Expecting error when spatial coordinates missing", {
        data(ClustSignal_example)
        samples <- "sample_id"
        colData(spe) <- cbind(colData(spe), spatialCoords(spe))
        spatialCoords(spe) <- NULL

        testthat::expect_error(
            suppressWarnings(clustSIGNAL(spe, samples))
        )
    }
)

#' Mouse Embryo Data
#'
#' This dataset contains spatial transcriptomics data from 3 mouse embryos, with
#' 351 genes and a total of 57536 cells. For vignettes, we subset the data by
#' randomly selecting 5000 cells from embryo 2, excluding cells that were
#' annotated as 'low quality'.
#'
#' @name mEmbryo2
#' @aliases me_data me_expr
#' @docType data
#' @format
#' \code{me_expr} a gene expression matrix with normalised counts, where rows
#' indicate genes and columns indicate cells.
#' \code{me_data} a data frame of cell metadata including cell IDs, sample IDs,
#' cell type annotations, and x-y coordinates of cells.
#' @usage data(mEmbryo2)
#' @source Integration of spatial and single-cell transcriptomic data elucidates
#' mouse organogenesis, \emph{Nature Biotechnology}, 2021.
#' Webpage: \url{https://www.nature.com/articles/s41587-021-01006-2}
#' @keywords datasets
NULL


#' Example data with SpatialExperiment object
#'
#' This example data was generated from the mouse embryo spatial transcriptomics
#' dataset of 3 mouse embryos, with 351 genes and a total of 57536 cells. For
#' running examples, we subset the data by selecting 1000 random cells from
#' embryo 2, excluding any cells annotated as 'low quality'. After subsetting,
#' we have expression for 351 genes from 1000 cells in embryo 2.
#'
#' @name ClustSignal_example
#' @aliases spe nnCells regXclust
#' @docType data
#' @format
#' \code{spe} a spatialExperiment object containing gene expression matrix with
#' normalised counts, where rows indicate genes and columns indicate cells.
#' Also, contains a cell metadata including cell IDs, sample IDs, cell type
#' annotations, and x-y coordinates of cells.
#' \code{nnCells} a matrix where each row corresponds to a cell in spe object,
#' and the columns correspond to the nearest neighbors.
#' \code{regXclust} a list where each element corresponds to a cell in spe
#' object, and contains the cluster composition proportions.
#' @usage data(ClustSignal_example)
#' @source Integration of spatial and single-cell transcriptomic data elucidates
#' mouse organogenesis, \emph{Nature Biotechnology}, 2021.
#' Webpage: \url{https://www.nature.com/articles/s41587-021-01006-2}
#' @keywords datasets
NULL


#' Mouse Hypothalamus Data
#'
#' This dataset contains spatial transcriptomics data from 181 mouse
#' hypothalamus samples, 155 genes and a total of 1,027,080 cells. For running
#' the vignettes, we subset the data by selecting total 6000 cells from only 3
#' samples - Animal 1 Bregma -0.09 (2080 cells) and Animal 7 Bregmas 0.16
#' (1936 cells) and -0.09 (1984 cells), excluding cells that were annotated as
#' 'ambiguous', and removed 20 genes that were assessed using a different
#' technology.
#'
#' @name mHypothal
#' @aliases mh_data mh_expr
#' @docType data
#' @format
#' \code{mh_expr} a gene expression matrix with normalised counts, where rows
#' indicate genes and columns indicate cells.
#' \code{mh_data} a data frame of cell metadata including cell IDs, sample IDs,
#' cell type annotations, and x-y coordinates of cells.
#' @usage data(mHypothal)
#' @source Molecular, Spatial and Functional Single-Cell Profiling of the
#' Hypothalamic Preoptic Region, \emph{Science}, 2018.
#' Webpage: \url{https://www.science.org/doi/10.1126/science.aau5324}
#' @keywords datasets
NULL

#' Cell neighbourhood detection
#'
#' @description
#' A function to identify the neighbourhood of each cell. If sort = TRUE, the
#' neighbourhoods are also sorted such that cells belonging to the same
#' 'initial cluster' as the index cell are arranged closer to it.
#' @param spe SpatialExperiment object with initial cluster and subcluster
#' labels.
#' @param samples a character indicating name of colData(spe) column containing
#' sample names.
#' @param NN an integer for the number of neighbouring cells the function
#' should consider. The value must be greater than or equal to 1. Default value
#' is 30.
#' @param sort a logical parameter for whether to sort the neighbourhood by
#' initial clusters. Default value is TRUE.
#' @param threads a numeric value for the number of CPU cores to be used for the
#' analysis. Default value set to 1.
#' @return a list containing two items:
#'
#' 1. nnCells, a character matrix of NN nearest neighbours - rows are index
#' cells and columns are their nearest neighbours ranging from closest to
#' farthest neighbour. For sort = TRUE, the neighbours belonging to the same
#' initial cluster as the index cell are moved closer to it.
#'
#' 2. regXclust, a list of vectors of each cell's neighbourhood composition
#' indicated by the proportion of initial subclusters it contains.
#' @importFrom BiocNeighbors findKNN KmknnParam
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom methods show
#' @examples
#' data(ClustSignal_example)
#'
#' out_list <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id")
#' out_list |> names()
#' @export

neighbourDetect <- function(spe, samples, NN = 30, sort = TRUE, threads = 1) {
    samplesList <- unique(spe[[samples]])
    nnCells <- matrix(nrow = 0, ncol = NN + 1)
    nnClusts <- matrix(nrow = 0, ncol = NN + 1)
    for (s in samplesList) {
        speX <- spe[, spe[[samples]] == s]
        xy_pos <- spatialCoords(speX)
        Clust <- as.data.frame(as.character(speX$initCluster))
        rownames(Clust) <- colnames(speX)
        subClust <- as.data.frame(speX$initSubcluster)
        rownames(subClust) <- colnames(speX)
        nnMatlist <- BiocNeighbors::findKNN(
            xy_pos, k = NN, num.threads = threads,
            BNPARAM = BiocNeighbors::KmknnParam())
        rownames(nnMatlist$index) <- rownames(subClust)
        nnMatlist$indexNew <- cbind(seq_len(nrow(nnMatlist$index)),
                                    nnMatlist$index)
        if (sort == TRUE) {
            nnCells <- rbind(nnCells, t(apply(nnMatlist$indexNew, 1,
                                              .cellNameSort, Clust = Clust)))
        } else if (sort == FALSE) {
            nnCells <- rbind(nnCells, t(apply(nnMatlist$indexNew, 1,
                                              .cellName, Clust = Clust)))}
        nnClusts <- rbind(nnClusts, t(apply(nnMatlist$indexNew, 1,
                                            .clustNum, subClust = subClust)))}
    regXclust <- apply(nnClusts, 1, .calculateProp)
    # QC check
    check.cells.dim <- identical(dim(nnCells),
                                 as.integer(c(ncol(spe), NN + 1)))
    check.cells.names <- identical(as.character(nnCells[,1]), colnames(spe))
    check.cells.NA <- sum(is.na(nnCells))
    check.clusts.dim <- identical(dim(nnClusts),
                                  as.integer(c(ncol(spe), NN + 1)))
    check.clusts.names <- identical(rownames(nnClusts), colnames(spe))
    check.clusts.NA <- sum(is.na(nnClusts))
    check.region <- identical(length(regXclust), ncol(spe))
    if (check.cells.dim == FALSE) {
        stop("Cell name issue - all neighbors were not detected for all cells.")
    } else if (check.cells.names == FALSE) {
        stop("Cell name issue - Cell order of neighbor data does not match cell
             order in input data.")
    } else if (check.clusts.dim == FALSE) {
        stop("Cluster number issue - all neighbors were not detected for all
             cells.")
    } else if (check.clusts.names == FALSE) {
        stop("Cluster number issues - Cell order of neighbor data does not match
             cell order in input data.")
    } else if (check.region == FALSE) {
        stop("Region proportions not calculated for all cells.")
    } else if (check.cells.NA != 0 | check.clusts.NA != 0) {
        stop("Missing values in neighbor data.")
    } else {
        show(paste("Regions defined. Time", format(Sys.time(),'%H:%M:%S')))}
    return(list("nnCells" = nnCells,
                "regXclust" = regXclust))
}

#' Initial non-spatial clustering
#'
#' @description
#' A function to perform initial non-spatial clustering and sub-clustering of
#' normalised gene expression to generate 'initial clusters' and 'initial
#' subclusters'.
#' @param spe a SpatialExperiment object containing spatial coordinates in
#' 'spatialCoords' matrix and normalised gene expression in 'logcounts' assay.
#' @param dimRed a character indicating the name of the reduced dimensions to
#' use from the SpatialExperiment object (i.e., from reducedDimNames(spe)).
#' Default value is 'None'.
#' @param batch a logical parameter for whether to perform batch correction.
#' Default value is FALSE.
#' @param batch_by a character indicating name of colData(spe) column containing
#' the batch names. Default value is 'None'.
#' @param threads a numeric value for the number of CPU cores to be used for the
#' analysis. Default value set to 1.
#' @param clustParams a list of parameters for TwoStepParam clustering methods:
#' clust_c is the number of centers to use for clustering with KmeansParam. By
#' default set to 0, in which case the method uses either 5000 centers or 1/5th
#' of the total cells in the data as the number of centers, whichever is lower.
#' subclust_c is the number of centers to use for sub-clustering the initial
#' clusters with KmeansParam. The default value is 0, in which case the method
#' uses either 1 center or half of the total cells in the initial cluster as
#' the number of centers, whichever is higher.
#' iter.max is the maximum number of iterations to perform during clustering
#' and sub-clustering with KmeansParam. Default value is 30.
#' k is a numeric value indicating the k-value used for clustering and
#' sub-clustering with NNGraphParam. Default value is 10.
#' cluster.fun is a character indicating the graph clustering method used with
#' NNGraphParam. By default, the Louvain method is used.
#' @return SpatialExperiment object with initial cluster and subcluster labels
#' of each cell.
#' @importFrom bluster clusterRows TwoStepParam KmeansParam NNGraphParam
#' @importFrom scater runPCA
#' @importFrom harmony RunHarmony
#' @importFrom SummarizedExperiment colData
#' @importFrom SingleCellExperiment reducedDim
#' @importFrom methods show
#' @importFrom stats setNames
#' @examples
#' data(ClustSignal_example)
#'
#' spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA")
#' spe$nsCluster |> head()
#' spe$initCluster |> head()
#' @export

p1_clustering <- function(spe, dimRed = "None", batch = FALSE,
                          batch_by = "None", threads = 1,
                          clustParams = list(clust_c = 0, subclust_c = 0,
                                             iter.max = 30, k = 10,
                                             cluster.fun = "louvain")) {
    if (clustParams[[1]] == 0)
        clustVal <- min(as.integer(ncol(spe) / 5), 5000) else
            clustVal <- clustParams[[1]]
    # Initial clustering
    if (batch == TRUE) {
        emb <- harmony::RunHarmony(data_mat = reducedDim(spe, dimRed),
                                   meta_data = colData(spe),
                                   vars_use = batch_by, max.iter = 20,
                                   verbose = FALSE)
        mat <- emb
    } else {
        mat <- reducedDim(spe, dimRed)
    }
    initClust <- bluster::clusterRows(mat, bluster::TwoStepParam(
        first = bluster::KmeansParam(centers = clustVal,
                                     iter.max = clustParams[[3]]),
        second = bluster::NNGraphParam(k = clustParams[[4]],
                                       cluster.fun = clustParams[[5]],
                                       num.threads = threads)))
    spe$initCluster <- factor(initClust)
    show(paste("Initial nonspatial clustering performed. Clusters =",
               length(unique(initClust)), "Time",
               format(Sys.time(),'%H:%M:%S')))
    # Initial subclustering
    clusters <- length(unique(spe$initCluster))
    subclusters_list <- list()
    for (cl in seq_len(clusters)){
        speX <- spe[, spe$initCluster == cl]
        speX <- scater::runPCA(speX)
        matX <- reducedDim(speX, "PCA")
        if (clustParams[[2]] == 0)
            subclustVal <- max(as.integer(ncol(speX) / 2), 1) else
                subclustVal <- clustParams[[2]]
        initClustX <- bluster::clusterRows(
            matX,
            bluster::TwoStepParam(
                first = bluster::KmeansParam(centers = subclustVal,
                                             iter.max = clustParams[[3]]),
                second = bluster::NNGraphParam(k = clustParams[[4]],
                                               cluster.fun = clustParams[[5]],
                                               num.threads = threads)))
        subclusters_list[[cl]] <- setNames(paste0(cl, ".", initClustX),
                                          colnames(speX))}
    spe$initSubcluster <- factor(unlist(subclusters_list)[colnames(spe)])
    show(paste("Nonspatial subclustering performed. Subclusters =",
               length(unique(spe$initSubcluster)), "Time",
               format(Sys.time(),'%H:%M:%S')))
    return (spe)
}

---
title: "ClustSIGNAL tutorial"
date: "`r Sys.Date()`"
author:
- name: Pratibha Panwar
  affiliation:
  - School of Mathematics and Statistics, The University of Sydney, NSW 2006, Australia; 
  - Sydney Precision Data Science Centre, University of Sydney, NSW 2006, Australia; 
  - Charles Perkins Centre, The University of Sydney, NSW 2006, Australia
- name: Boyi Guo
  affiliation:
  - Department of Biostatistics, Johns Hopkins Bloomberg School of Public Health, MD, USA
- name: Haowen Zhou
  affiliation:
  - Bioinformatics and Systems Biology Graduate Program, University of California San Diego, La Jolla, CA, USA
- name: Stephanie Hicks
  affiliation:
  - Department of Biostatistics, Johns Hopkins Bloomberg School of Public Health, MD, USA; 
  - Department of Biomedical Engineering, Johns Hopkins University, Baltimore, MD, USA; 
  - Center for Computational Biology, Johns Hopkins University, Baltimore, MD, USA; 
  - Malone Center for Engineering in Healthcare, Johns Hopkins University, MD, USA
- name: Shila Ghazanfar
  affiliation:
  - School of Mathematics and Statistics, The University of Sydney, NSW 2006, Australia; 
  - Sydney Precision Data Science Centre, University of Sydney, NSW 2006, Australia; 
  - Charles Perkins Centre, The University of Sydney, NSW 2006, Australia
output:
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: default
package: clustSIGNAL
vignette: |
    %\VignetteIndexEntry{ClustSIGNAL tutorial}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 80
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = FALSE, warning = FALSE)
```

# ClustSIGNAL

The R package ClustSIGNAL performs spatially-informed cell type clustering on 
high-resolution spatial transcriptomics data. It uses both the gene expression 
and spatial locations of cells to group them into clusters.

## Motivation

ClustSIGNAL aims to: (i) overcome data sparsity using an adaptive smoothing 
approach that is guided by the heterogeneity/homogeneity of each individual 
cell's neighbourhood; (ii) embed spatial context information into the gene 
expression generating a transformed, adaptively smoothed expression matrix that 
can be used for clustering; and (iii) generate entropy data that captures the
heterogeneity/homogeneity information from each cell's neighbourhood and can be 
used to create a spatial map of heterogeneity distribution in a sample tissue.

## Overview

In this vignette, we demonstrate how spatially-informed clustering can be
performed with ClustSIGNAL, assessing the clusters using pre-defined metrics
like [adjusted rand index (ARI)](https://www.rdocumentation.org/packages/aricode/versions/0.1.1/topics/ARI) 
and [normalized mutual information (NMI)](https://www.rdocumentation.org/packages/aricode/versions/0.1.1/topics/NMI) 
from the [aricode](https://jmlr.csail.mit.edu/papers/volume11/vinh10a/vinh10a.pdf) 
R package, as well as spatial plots to visualize them. ClustSIGNAL is a 
multisample spatial clustering approach, and we show this using an example 
dataset. We also display the use of entropy values, which are generated as part 
of the ClustSIGNAL process, in understanding the tissue structure of a sample. 

ClustSIGNAL is very flexible in that it allows for, (i) user-provided input 
values for most parameters (default parameter values are also provided) and 
(ii) running ClustSIGNAL step-by-step. This tutorial demonstrates how the 
step-by-step clustering can be performed, and what parameters need to be 
defined at each step.

```{r load_packages, message = FALSE, warning = FALSE}
# load required packages
library(clustSIGNAL)
library(scater)
library(ggplot2)
library(dplyr)
library(patchwork)
library(aricode)
```

# Single sample analysis with ClustSIGNAL

In this section, we use the SeqFISH mouse embryo dataset from [Lohoff et al,
2021](https://www.nature.com/articles/s41587-021-01006-2), which contains
spatial transcriptomics data from 3 mouse embryos, with 351 genes and 57,536 
cells. For this vignette, we have subset the data by randomly selecting 5000
cells from Embryo 2, excluding cells that had been manually annotated as 'Low
quality'.

We begin by creating a SpatialExperiment object from the gene expression and
cell information in the data subset, ensuring that the spatial coordinates are
stored in spatialCoords within the SpatialExperiment object. If the data are
already in a SpatialExperiment object, ClustSIGNAL can be run as long as basic 
requirements like spatial coordinates, normalized counts, and unique cell names 
are met.

```{r embryo_data_prep}
# load me_expr containing gene expression logcounts
# load me_data containing cell metadata including x-y coordinates
data(mEmbryo2)
# to create a SpatialExperiment object we need gene expression, cell metadata, 
# and cell locations.
spe <- SpatialExperiment::SpatialExperiment(
  assays = list(logcounts = me_expr), 
  colData = me_data,
  # spatialCoordsNames requires column names in me_data that contain 
  # xy-coordinates of cells
  spatialCoordsNames = c("X", "Y"))
spe
```

For running ClustSIGNAL, we need to know the column name in colData slot of the
SpatialExperiment object that contains the sample labels. Here, the sample 
labels are in the 'sample_id' column.

```{r embryo_data_columns}
spe |> colData() |> colnames() # column names in the metadata
```

## Running ClustSIGNAL on one sample

The simplest ClustSIGNAL run requires a SpatialExperiment object, the colData 
column name of sample labels, and the type of output to generate. 

Other parameters that can be modified include: (i) dimRed - specifies the low 
dimension data to use (default 'None'); (ii) batch - when TRUE, ClustSIGNAL 
performs batch correction and needs a valid value for batch_by; (iii) batch_by - 
name of metadata column containing sample batches contributing to batch effect 
(default 'None'); (iv) NN - specifies the neighbourhood size (default 30); 
(v) kernel - specifies the distribution to use for weight generation (default 
'G' for Gaussian); (vi) spread - the distribution spread value (default 0.3 for 
Gaussian); (vii) sort - when TRUE, ClustSIGNAL sorts the neighbourhood; (viii) 
threads - specifies the number of cpus to use in parallel runs (default 1); and 
(ix) clustParams - list of parameters to use for non-spatial clustering 
components.

Furthermore, the adaptively smoothed gene expression data generated by
ClustSIGNAL could be useful for other downstream analyses and is accessible if 
the output options 's' or 'a' are selected to return the final 
SpatialExperiment object.

```{r ClustSIGNAL_singleRun}
set.seed(100)
samples <- "sample_id" # column name containing sample names
# to run ClustSIGNAL, requires a SpatialExperiment object, column name of sample
# labels in colData slot, and the output type to generate (clusters, neighbours,
# and/or final spe object).
res_emb <- clustSIGNAL(spe, samples, outputs = "a") 
```

This returns a list that contains a ClustSIGNAL clusters dataframe (clusters), 
a matrix of cell IDs from each cell's neighbourhood (neighbours with NN 
neighbourhood size), and a final SpatialExperiment object (spe_final).

```{r embryo_result_list}
res_emb |> names() # names of the outputs generated
```

The cluster dataframe contains cell IDs and their cluster labels assigned by 
ClustSIGNAL.

```{r embryo_clusters_head}
res_emb$clusters |> head() # cluster data frame has cell IDs and cluster labels
```

The output SpatialExperiment object contains the adaptively smoothed gene
expression data as an additional assay (smoothed), as well as initial clusters 
and subclusters, entropy values, and ClustSIGNAL clusters.

```{r embryo_final_spe}
# for convenience with downstream analyses, we will replace the original spe
# object with the one generated by ClustSIGNAL. This does not lead to any loss 
# of information as ClustSIGNAL only adds information to the input spe object.
spe <- res_emb$spe_final
spe
spe |> colData() |> colnames()
```

## Visualising ClustSIGNAL clusters

We use spatial coordinates of cells and their ClustSIGNAL cluster labels and 
entropy values to visualize the clustering output.

```{r colors}
colors <- c("#635547", "#8EC792", "#9e6762", "#FACB12", "#3F84AA", "#0F4A9C", 
            "#ff891c", "#EF5A9D", "#C594BF", "#DFCDE4", "#139992", "#65A83E", 
            "#8DB5CE", "#005579", "#C9EBFB", "#B51D8D", "#532C8A", "#8870ad", 
            "#cc7818", "#FBBE92", "#EF4E22", "#f9decf", "#c9a997", "#C72228", 
            "#f79083", "#F397C0", "#DABE99", "#c19f70", "#354E23", "#C3C388",
            "#647a4f", "#CDE088", "#f7f79e", "#F6BFCB", "#7F6874", "#989898", 
            "#1A1A1A", "#FFFFFF", "#e6e6e6", "#77441B", "#F90026", "#A10037", 
            "#DA5921", "#E1C239", "#9DD84A")
```

```{r embryo_spatialPlots1}
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension slot of the spe object
reducedDim(spe, "spatial") <- spatialCoords(spe)
```

```{r embryo_spatialPlots2}
# spatial plot
spt_clust <- scater::plotReducedDim(
  spe, colour_by = "ClustSIGNAL", dimred = "spatial", point_alpha = 1,
  point_size = 4, scattermore = TRUE) +
  ggtitle("A. Spatial plot of clusters") +
  scale_color_manual(values = colors) +
  guides(colour = guide_legend(title = "Clusters", 
                               override.aes = list(size = 5))) +
  theme(text = element_text(size = 12))
```

```{r embryo_spatialPlots3}
# entropy distribution plotted at cluster-level can indicate which clusters 
# have cells from homogeneous/heterogeneous space. 
df_met <- spe |> colData() %>% as.data.frame()
ct_ent <- df_met %>% 
  mutate(ClustSIGNAL = as.character(ClustSIGNAL)) %>%
  group_by(ClustSIGNAL) %>%
  # calculating median entropy of each cluster category
  summarise(mdEntropy = median(entropy)) %>% 
  # reordering clusters by their median entropy value
  arrange(mdEntropy)
df_met$ClustSIGNAL <- factor(df_met$ClustSIGNAL, levels = ct_ent$ClustSIGNAL)
col_ent <- colors[as.numeric(as.character(ct_ent$ClustSIGNAL))]
box_clust <- df_met %>%
  ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
  geom_boxplot() +
  scale_fill_manual(values = col_ent) +
  ggtitle("B. Entropy distribution of clusters") +
  labs(x = "ClustSIGNAL clusters", y = "Entropy", name = "Clusters") +
  theme_classic() +
  theme(legend.position = "none",
        text = element_text(size = 12),
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1),
        plot.title = element_text(face = "bold"))
```

```{r embryo_spatialPlots4}
spt_clust + box_clust + patchwork::plot_layout(guides = "collect", 
                                               widths = c(2, 3))
```

The spatial location and entropy distribution of the clusters provide
spatial context of the cells and their neighbourhoods, as well as the
compositions of the neighbourhoods. For example, in panel (B) the low entropy 
clusters are generally found in space that is more homogeneous, whereas the 
high entropy clusters belong to neighbourhoods that have more cell diversity. 
This can also be visualized in the spatial plot in panel (A).

## Assessing clustering accuracy

We assess the clustering efficiency of ClustSIGNAL using the commonly used
clustering metrics ARI and NMI, which are usable only when prior cell
annotations are available. Here, ARI and NMI measure the similarity or 
agreement (respectively) between cluster labels obtained from ClustSIGNAL and 
manual cell annotations.

```{r embryo_clusterMetrics}
# to assess the accuracy of clustering, the cluster labels are often compared to
# prior annotations. Here, we compare ClustSIGNAL cluster labels to annotations 
# available with this public data.
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(
    ARI = aricode::ARI(celltype_mapped_refined, ClustSIGNAL), # calculate ARI
    NMI = aricode::NMI(celltype_mapped_refined, ClustSIGNAL)) # calculate NMI
```

## Entropy spread and distribution

The entropy values generated through ClustSIGNAL process can be useful in
analyzing the sample structure. 

```{r embryo_entropyMetrics}
# we can assess the overall entropy distribution of the dataset
spe |> colData() %>% 
  as.data.frame() %>%
  summarise(min_Entropy = min(entropy),
            min_Entropy_count = sum(spe$entropy == 0),
            max_Entropy = max(entropy),
            mean_Entropy = mean(entropy))
```

The entropy range can indicate whether the tissue sample contains any 
homogeneous regions. For example, a min_Entropy of 0 means that some cells are 
placed in completely homogeneous space when looking at a neighbourhood size of 
30 cells (NN = 30 was used for generating the entropy values). The 
min_Entropy_count gives us an idea of the total number of such low entropy 
neighbourhoods in the sample.

```{r entropyPlots1}
# we can also visualize the distribution and spread of the entropy values
hst_ent <- spe |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  ggtitle("A. Entropy spread") +
  labs(x = "Entropy", y = "Number of neighbourhoods") +
  theme_classic() +
  theme(text = element_text(size = 12),
        plot.title = element_text(face = "bold"))
```

```{r entropyPlots2}
spt_ent <- scater::plotReducedDim(spe, colour_by = "entropy",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  ggtitle("B. Entropy spatial distribution") +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe$entropy))) +
  theme(text = element_text(size = 12))
```

```{r entropyPlots3}
hst_ent + spt_ent
```

The spread and spatial distribution of neighbourhood entropies can be useful in 
visually assessing and comparing tissue compositions in samples - low entropy 
neighbourhoods are more homogeneous and likely contain cell type-specific 
niches, whereas high entropy neighbourhoods are heterogeneous with more uniform 
distribution of different cell types.

# Multisample analysis with ClustSIGNAL

Here, we use the MERFISH mouse hypothalamus preoptic region dataset from
[Moffitt et al, 2018](https://www.science.org/doi/10.1126/science.aau5324),
which contains spatial transcriptomics data from 181 samples, with 155 genes 
and 1,027,080 cells. For this vignette, we have subset the data by selecting 
6000 random cells from only 3 samples - Animal 1 Bregma -0.09 (2080 cells), 
Animal 7 Bregma 0.16 (1936 cells), and Animal 7 Bregma -0.09 (1984 cells), 
excluding cells that were manually annotated as 'Ambiguous' and 20 genes for 
which expression was generated using a different technology.

We start the analysis by creating a SpatialExperiment object from the gene
expression and cell information in the data subset, ensuring that the spatial
coordinates are stored in spatialCoords slot within the spe object.

```{r hypothal_data_prep}
# load mh_expr containing gene expression logcounts
# load mh_data containing cell metadata and cell x-y coordinates
data(mHypothal)
# create spe object using gene expression, cell metadata, and cell locations
spe2 <- SpatialExperiment(assays = list(logcounts = mh_expr), 
                          colData = mh_data,
                          # spatialCoordsNames requires column names in 
                          # mh_data that contain xy-coordinates of cells
                          spatialCoordsNames = c("X", "Y"))
spe2
```

Next we identify sample labels column in the SpatialExperiment object.

```{r hypothal_data_columns}
spe2 |> colData() |> str() # metadata summary
```

Here, the sample labels are in the ‘samples’ column of the object.

## ClustSIGNAL run

An important concept to take into account when running multisample analysis is 
batch effects. When gathering samples from different sources or through 
different technologies/procedures, some technical batch effects might be 
introduced into the dataset. We can run ClustSIGNAL in batch correction mode
simply by setting batch = TRUE and batch_by = "group", where group will be the 
name of the colData column of spe object that contains the batch information. 
ClustSIGNAL then uses [harmony](https://portals.broadinstitute.org/harmony/) 
internally for batch correction.

```{r ClustSIGNAL_multiRun }
set.seed(110)
# ClustSIGNAL can be run on a dataset with multiple samples. As before, we need
# the SpatialExperiment object and column name of sample labels in the object. 
# The method can be run in parallel through the threads option. Here we use 
# thread = 4 to use 4 cores.
# Since no batch effects were observed in this data subset, we have not used 
# the batch and batch_by options.
samples <- "samples" # column name containing sample names
res_hyp <- clustSIGNAL(spe2, samples, threads = 4, outputs = "a")
```

```{r hypothal_final_spe}
# for convenience with downstream analyses, we replace the original spe object 
# with the one generated by ClustSIGNAL.
spe2 <- res_hyp$spe_final
spe2
```

## Clustering metrics

Clustering and entropy results can be calculated and visualized for each sample.

```{r hypothal_samples}
samplesList <- spe2[[samples]] |> levels() # get sample names
samplesList
```

```{r hypothal_clusterMetrics}
spe2 |> colData() %>% 
  as.data.frame() %>%
  group_by(samples) %>%
  summarise(
    # Comparing ClustSIGNAL cluster labels to annotations available with the 
    # public data to assess its accuracy.
    ARI = aricode::ARI(Cell_class, ClustSIGNAL),
    NMI = aricode::NMI(Cell_class, ClustSIGNAL),
    # Assessing the overall entropy distribution of the samples in the dataset.
    min_Entropy = min(entropy),
    min_Entropy_count = sum(entropy == 0),
    max_Entropy = max(entropy),
    mean_Entropy = mean(entropy))
```

As before, the entropy range can tell us a lot about the tissue structure of 
the samples. Unlike the seqFISH subset data, where the minimum entropy of the 
sample was 0, here, the minimum entropy is higher indicating that the tissue 
doesn't really have any cell type-specific niches when looking at neighbourhood 
size of 30 cells. Moreover, the relatively high mean entropy value indicates 
that the tissues slices are quite heterogeneous.

## Visualizing ClustSIGNAL clusters

ClustSIGNAL performs clustering on all cells in the dataset in one run, thereby
generating the same clusters across multiple samples. The cluster labels do not 
need to be mapped between samples. For example, cluster 1 represents the same
cell type in all three samples, without needing explicit mapping between 
samples.

```{r hypothal_spatialPlots1}
# for plotting with scater R package, we need to add the spatial coordinates 
# to the reduced dimension section
reducedDim(spe2, "spatial") <- spatialCoords(spe2)
```

```{r hypothal_spatialPlots2}
# spatial plot - ClustSIGNAL clusters
spt_clust2 <- scater::plotReducedDim(spe2, colour_by = "ClustSIGNAL",
                                    # specify spatial low dimension
                                    dimred = "spatial", point_alpha = 1,
                                    point_size = 4, scattermore = TRUE) +
  scale_color_manual(values = colors) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  guides(colour = guide_legend(title = "Clusters",
                               override.aes = list(size = 3))) +
  theme(text = element_text(size = 12))
```

```{r hypothal_spatialPlots3}
# For visualising cluster-level entropy distribution, we reorder the clusters 
# by their median entropy value in each sample
df_met2 <- spe2 |> colData() %>% as.data.frame()

box_clust2 <- list()
for (s in samplesList) {
  df_met_sub <- df_met2[df_met2[[samples]] == s, ]
  # calculating median entropy of each cluster in a sample
  ct_ent2 <- df_met_sub %>%
    mutate(ClustSIGNAL = as.character(ClustSIGNAL)) %>%
    group_by(ClustSIGNAL) %>%
    summarise(mdEntropy = median(entropy)) %>%
    # reordering clusters by their median entropy
    arrange(mdEntropy)
  
  df_met_sub$ClustSIGNAL <- factor(df_met_sub$ClustSIGNAL, 
                                   levels = ct_ent2$ClustSIGNAL)
  # box plot of cluster entropy
  col_ent2 <- colors[as.numeric(ct_ent2$ClustSIGNAL)]
  box_clust2[[s]] <- df_met_sub %>%
    ggplot(aes(x = ClustSIGNAL, y = entropy, fill = ClustSIGNAL)) +
    geom_boxplot() +
    scale_fill_manual(values = col_ent2) +
    facet_wrap(vars(samples), nrow = 1) +
    labs(x = "ClustSIGNAL clusters", y = "Entropy") +
    ylim(0, NA) +
    theme_classic() +
    theme(strip.text = element_blank(),
          legend.position = "none",
          text = element_text(size = 12),
          axis.text.x = element_text(angle = 90, vjust = 0.5))
}
```

```{r hypothal_spatialPlots4}
spt_clust2 / (patchwork::wrap_plots(box_clust2[1:3], nrow = 1) +
                plot_layout(axes = "collect")) +
  plot_layout(guides = "collect", heights = c(5, 3)) +
  plot_annotation(
    title = "Spatial (top) and entropy (bottom) distributions of clusters",
    theme = theme(plot.title = element_text(hjust = 0.5, face = "bold")))
```

The spatial location and entropy distribution of the clusters can be compared 
in a multisample analysis, providing spatial context of the cluster cells and 
their neighbourhood compositions in the different samples within the dataset.
Since the clusters were generated in a single run, they are same across the 
different samples. Therefore, if a cluster is not represented in a sample, this 
would mean that its respective cell type is not present in that sample.  

## Visualising entropy spread and distribution

In multisample analysis, tissue structure of the different samples in the 
dataset can be compared using the spread and spatial distribution of the 
neighbourhood entropy measures.

```{r hypothal_entropyPlots1}
hst_ent2 <- spe2 |> colData() %>% 
  as.data.frame() %>%
  ggplot(aes(entropy)) +
  geom_histogram(binwidth = 0.05) +
  facet_wrap(vars(samples), nrow = 1) +
  labs(x = "Entropy", y = "Number of neighbourhoods") +
  theme_classic() +
  theme(text = element_text(size = 12))
```

```{r hypothal_entropyPlots2}
spt_ent2 <- scater::plotReducedDim(spe2, colour_by = "entropy",
                                  # specify spatial low dimension
                                  dimred = "spatial", point_alpha = 1,
                                  point_size = 4, scattermore = TRUE) +
  scale_colour_gradient2("Entropy", low = "grey", high = "blue") +
  scale_size_continuous(range = c(0, max(spe2$entropy))) +
  facet_wrap(vars(spe2[[samples]]), scales = "free", nrow = 1) +
  theme(strip.text = element_blank(),
        text = element_text(size = 12))
```

```{r hypothal_entropyPlots3}
hst_ent2 / spt_ent2 + plot_layout(heights = c(4, 5)) +
    plot_annotation(
      title = "Entropy spread (top) and spatial distribution (bottom)",
      theme = theme(plot.title = element_text(hjust = 0.5, face = "bold")))
```

Together, these plots help in visually assessing tissue compositions of the 
samples - all 3 samples have high entropy neighbourhoods indicating that they 
mainly have heterogeneous regions with uniform distribution of different cell 
types.

# ClustSIGNAL step-by-step run

ClustSIGNAL has five main functions for each distinct step in its algorithm.
These functions are accessible and can be run sequentially to generate data 
from intermediate steps, if needed. For example, ClustSIGNAL can be run
step-by-step up to the entropy measurement component, without having to run the
complete method. The entropy values will be added to the SpatialExperiment
object and can be used for assessing tissue structure in terms of its
heterogeneity. Similarly, the adaptively smoothed gene expression can be 
obtained by running ClustSIGNAL till the adaptive smoothing step. Here, we 
describe how individual ClustSIGNAL functions can be used sequentially.

```{r ClustSIGNALseq_data}
# load logcounts and metadata to the environment
data(mEmbryo2)

# as before, we read the data into a SpatialExperiment object
spe <- SpatialExperiment(assays = list(logcounts = me_expr),
                         colData = me_data, spatialCoordsNames = c("X", "Y"))
```

```{r ClustSIGNALseq_prep}
set.seed(100)
# first we need to generate low dimension data for initial clustering
spe <- scater::runPCA(spe) 
```

## Step 1: Initial clustering and subclustering

The first step in the ClustSIGNAL algorithm is initial clustering and
subclustering. For this, we need to provide a spe object with low embedding 
information. 

Other parameters have default values: batch = FALSE and batch_by = "None" (if 
no batch correction needs to be performed), threads = 1, 
clustParams = list(clust_c = 0, subclust_c = 0, iter.max = 30, 
k = 10, cluster.fun = "louvain").

Among the clustering parameters, clust_c and subclust_c refer to the number of 
centers to use for clustering and sub-clustering with KmeansParam. By default 
clust_c is set to 0, in which case the method uses either 5000 centers or 1/5th
of the total cells in the data as the number of centers, whichever is lower.
Similarly, subclust_c is set to 0 by default, in which case the method
uses either 1 center or half of the total cells in the initial cluster as
the number of centers, whichever is higher. For all other values of clust_c and 
subclust_c, the input is treated as the number of centers.

```{r ClustSIGNALseq_step1}
spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA")
```

Here, two columns are added to the spe object under the cell metadata:

(i) the initial cluster labels,

```{r ClustSIGNALseq_step1_out1}
spe$initCluster |> head() # clustering output
```

(ii) the initial subcluster labels.

```{r ClustSIGNALseq_step1_out2}
spe$initSubcluster |> head() # subclustering output
```

## Step 2: Neighbourhood detection

The next step involves detecting the neighborhood of all cells. We need the spe 
object containing the initial cluster and initial subcluster labels and sample 
IDs for this. 

By default, ClustSIGNAL identifies 30 nearest neighbors (NN = 30), sorts the 
neighbourhood (sort = TRUE), and does not use parallel runs (threads = 1).

ClustSIGNAL allows the use of external cell labels generated through other 
methods, in place of the initial clusters and subclusters. For this, the 
cell cluster and subcluster labels of each cell must be stored in the colData 
of the spe object as "initCluster" and "initSubcluster", respectively.

```{r ClustSIGNALseq_step2}
# This step generates a list of neighbourhood information.
outReg <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id")
```

This generates a list containing:

(i) a neighborhood matrix containing cell IDs,

```{r ClustSIGNALseq_step2_out1}
outReg$nnCells[1:3, 1:3]
```

(ii) a list of arrays containing initial subcluster proportions.

```{r ClustSIGNALseq_step2_out2}
outReg$regXclust[[1]] 
```

## Step 3: Entropy measure

Now that we know the neighbourhood of each cell, we can calculate entropy of 
each cell's neighborhood. For this, we need the spe object and initial 
subcluster proportions. This step can run in parallel, but by default we use 
1 cpu core.

```{r ClustSIGNALseq_step3}
spe <- clustSIGNAL::entropyMeasure(spe, outReg$regXclust)
```

The entropy values are added to the spe object under cell metadata.

```{r ClustSIGNALseq_step3_out}
spe$entropy |> head() # entropy values
```

## Step 4: Adaptive smoothing

Using the entropy values, we can perform adaptive smoothing. This requires the 
spe object containing the entropy values as well as the neighborhood matrix of 
cell IDs generated during neighbourhood detection. 

Other parameters for which default values are provided include number of 
neighbors (NN = 30), weight distribution type (kernel = "G" for Gaussian), 
distribution spread (spread = 0.05 representing standard deviation for Gaussian 
distribution; for exponential distribution we recommend using a spread of 5 
indicating rate of the distribution), and number of cores (threads = 1) to use 
for parallel runs.

```{r ClustSIGNALseq_step4}
spe <- clustSIGNAL::adaptiveSmoothing(spe, outReg$nnCells)
```

The adaptively smoothed gene expression data are added to the spe object under 
assays as 'smoothed'.

```{r ClustSIGNALseq_step4_out}
assay(spe, "smoothed")[1:5, 1:3]
```

## Step 5: Final clustering

The final step involves performing clustering on the adaptively smoothed data.
We only need to provide the spe object containing the adaptively smoothed
data. This step has the same default clustering and batch correction parameters 
as the initial clustering in first step.

```{r ClustSIGNALseq_step5}
spe <- clustSIGNAL::p2_clustering(spe)
```

Cluster labels are added to the colData of the spe object under a ClustSIGNAL 
column

```{r ClustSIGNALseq_step5_out}
spe$ClustSIGNAL |> head() # ClustSIGNAL cluster labels
```

<details>

<summary>**Session Information**</summary>

```{r}
sessionInfo()
```

</details>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ClustSignal_example}
\alias{ClustSignal_example}
\alias{spe}
\alias{nnCells}
\alias{regXclust}
\title{Example data with SpatialExperiment object}
\format{
\code{spe} a spatialExperiment object containing gene expression matrix with
normalised counts, where rows indicate genes and columns indicate cells.
Also, contains a cell metadata including cell IDs, sample IDs, cell type
annotations, and x-y coordinates of cells.
\code{nnCells} a matrix where each row corresponds to a cell in spe object,
and the columns correspond to the nearest neighbors.
\code{regXclust} a list where each element corresponds to a cell in spe
object, and contains the cluster composition proportions.
}
\source{
Integration of spatial and single-cell transcriptomic data elucidates
mouse organogenesis, \emph{Nature Biotechnology}, 2021.
Webpage: \url{https://www.nature.com/articles/s41587-021-01006-2}
}
\usage{
data(ClustSignal_example)
}
\description{
This example data was generated from the mouse embryo spatial transcriptomics
dataset of 3 mouse embryos, with 351 genes and a total of 57536 cells. For
running examples, we subset the data by selecting 1000 random cells from
embryo 2, excluding any cells annotated as 'low quality'. After subsetting,
we have expression for 351 genes from 1000 cells in embryo 2.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalClustering.R
\name{p2_clustering}
\alias{p2_clustering}
\title{Final non-spatial clustering}
\usage{
p2_clustering(
  spe,
  batch = FALSE,
  batch_by = "None",
  threads = 1,
  clustParams = list(clust_c = 0, subclust_c = 0, iter.max = 30, k = 10, cluster.fun =
    "louvain")
)
}
\arguments{
\item{spe}{SpatialExperiment object containing the adaptively smoothed gene
expression.}

\item{batch}{a logical parameter for whether to perform batch correction.
Default value is FALSE.}

\item{batch_by}{a character indicating name of colData(spe) column containing
the batch names. Default value is 'None'.}

\item{threads}{a numeric value for the number of CPU cores to be used for the
analysis. Default value set to 1.}

\item{clustParams}{a list of parameters for TwoStepParam clustering methods:
clust_c is the number of centers to use for clustering with KmeansParam. By
default set to 0, in which case the method uses either 5000 centers or 1/5th
of the total cells in the data as the number of centers, whichever is lower.
subclust_c is the number of centers to use for sub-clustering the initial
clusters with KmeansParam. This parameter is not used in the final
clustering step.
iter.max is the maximum number of iterations to perform during clustering
and sub-clustering with KmeansParam. Default value is 30.
k is a numeric value indicating the k-value used for clustering with
NNGraphParam. Default value is 10.
cluster.fun is a character indicating the graph clustering method used with
NNGraphParam. By default, the Louvain method is used.}
}
\value{
SpatialExperiment object containing clusters generated from smoothed
data.
}
\description{
A function to perform clustering on adaptively smoothed gene expression data
to generate ClustSIGNAL clusters.
}
\examples{
data(ClustSignal_example)

# For non-spatial clustering of normalised counts
spe <- clustSIGNAL::p2_clustering(spe)
spe$ClustSIGNAL |> head()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.calculateProp}
\alias{.calculateProp}
\title{Cell neighbourhood composition}
\usage{
.calculateProp(arr)
}
\arguments{
\item{arr}{a vector of initial subcluster labels of each cell in the
neighbourhood.}
}
\value{
a table of initial subcluster proportions in a neighbourhood.
}
\description{
A function to calculate the cell neighbourhood composition of initial cluster
labels.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustSIGNAL.R
\name{clustSIGNAL}
\alias{clustSIGNAL}
\title{ClustSIGNAL}
\usage{
clustSIGNAL(
  spe,
  samples,
  dimRed = "None",
  batch = FALSE,
  batch_by = "None",
  NN = 30,
  kernel = "G",
  spread = 0.3,
  sort = TRUE,
  threads = 1,
  outputs = "c",
  clustParams = list(clust_c = 0, subclust_c = 0, iter.max = 30, k = 10, cluster.fun =
    "louvain")
)
}
\arguments{
\item{spe}{a SpatialExperiment object containing spatial coordinates in
'spatialCoords' matrix and normalised gene expression in 'logcounts' assay.}

\item{samples}{a character indicating name of colData(spe) column containing
sample names.}

\item{dimRed}{a character indicating the name of the reduced dimensions to
use from the SpatialExperiment object (i.e., from reducedDimNames(spe)).
Default value is 'None'.}

\item{batch}{a logical parameter for whether to perform batch correction.
Default value is FALSE.}

\item{batch_by}{a character indicating name of colData(spe) column containing
the batch names. Default value is 'None'.}

\item{NN}{an integer for the number of neighbouring cells the function
should consider. The value must be greater than or equal to 1. Default value
is 30.}

\item{kernel}{a character for type of distribution to be used. The two valid
values are "G" or "E" for Gaussian and exponential distributions,
respectively. Default value is "G".}

\item{spread}{a numeric value for distribution spread, represented by
standard deviation for Gaussian distribution and rate for exponential
distribution. Default value is 0.3 for Gaussian distribution. The recommended
value is 5 for exponential distribution.}

\item{sort}{a logical parameter for whether to sort the neighbourhood by
initial clusters. Default value is TRUE.}

\item{threads}{a numeric value for the number of CPU cores to be used for the
analysis. Default value set to 1.}

\item{outputs}{a character for the type of output to return to the user. "c"
for data frame of cell IDs and their respective ClustSIGNAL cluster labels,
"n" for ClustSIGNAL cluster dataframe plus neighbourhood matrix, "s" for
ClustSIGNAL cluster dataframe plus final SpatialExperiment object, or "a" for
all 3 outputs.}

\item{clustParams}{a list of parameters for TwoStepParam clustering methods:
clust_c is the number of centers to use for clustering with KmeansParam. By
default set to 0, in which case the method uses either 5000 centers or 1/5th
of the total cells in the data as the number of centers, whichever is lower.
subclust_c is the number of centers to use for sub-clustering the initial
clusters with KmeansParam. The default value is 0, in which case the method
uses either 1 center or half of the total cells in the initial cluster as
the number of centers, whichever is higher.
iter.max is the maximum number of iterations to perform during clustering
and sub-clustering with KmeansParam. Default value is 30.
k is a numeric value indicating the k-value used for clustering and
sub-clustering with NNGraphParam. Default value is 10.
cluster.fun is a character indicating the graph clustering method used with
NNGraphParam. By default, the Louvain method is used.}
}
\value{
a list of outputs depending on the type of outputs specified in the
main function call.

1. clusters: a data frame of cell names and their ClustSIGNAL cluster
classification.

2. neighbours: a character matrix containing cells IDs of each cell's
NN neighbours.

3. spe_final: a SpatialExperiment object containing the original spe object
data plus initial cluster and subcluster labels, entropy values, smoothed
gene expression, and ClustSIGNAL cluster labels.
}
\description{
A clustering method for spatially-resolved cell-type classification of
spatial transcriptomics data. The tool generates and uses an adaptively
smoothed, spatially informed gene expression data for clustering.
}
\examples{
data(ClustSignal_example)

names(colData(spe))
# identify the column name with sample labels
samples = "sample_id"
res_list <- clustSIGNAL(spe, samples, outputs = "c")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.clustNum}
\alias{.clustNum}
\title{Neighbour cell initial subcluster label}
\usage{
.clustNum(cell, subClust)
}
\arguments{
\item{cell}{a vector of neighbourhood cell indices. The cell indices indicate
the row number of cells in sample metadata.}

\item{subClust}{a data frame of initial subcluster labels of each cell in
the sample.}
}
\value{
a data frame of initial subcluster labels of neighbourhood
cells.
}
\description{
A function to fetch initial subcluster label of cell.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neighborDetect.R
\name{neighbourDetect}
\alias{neighbourDetect}
\title{Cell neighbourhood detection}
\usage{
neighbourDetect(spe, samples, NN = 30, sort = TRUE, threads = 1)
}
\arguments{
\item{spe}{SpatialExperiment object with initial cluster and subcluster
labels.}

\item{samples}{a character indicating name of colData(spe) column containing
sample names.}

\item{NN}{an integer for the number of neighbouring cells the function
should consider. The value must be greater than or equal to 1. Default value
is 30.}

\item{sort}{a logical parameter for whether to sort the neighbourhood by
initial clusters. Default value is TRUE.}

\item{threads}{a numeric value for the number of CPU cores to be used for the
analysis. Default value set to 1.}
}
\value{
a list containing two items:

1. nnCells, a character matrix of NN nearest neighbours - rows are index
cells and columns are their nearest neighbours ranging from closest to
farthest neighbour. For sort = TRUE, the neighbours belonging to the same
initial cluster as the index cell are moved closer to it.

2. regXclust, a list of vectors of each cell's neighbourhood composition
indicated by the proportion of initial subclusters it contains.
}
\description{
A function to identify the neighbourhood of each cell. If sort = TRUE, the
neighbourhoods are also sorted such that cells belonging to the same
'initial cluster' as the index cell are arranged closer to it.
}
\examples{
data(ClustSignal_example)

out_list <- clustSIGNAL::neighbourDetect(spe, samples = "sample_id")
out_list |> names()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.generateBPParam}
\alias{.generateBPParam}
\title{Generating BPParam object}
\usage{
.generateBPParam(cores = 1)
}
\arguments{
\item{cores}{Desired number of cores for BPPARAM object.}
}
\value{
A BPPPARAM object.
}
\description{
A utility function to generate BPPARAM object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mHypothal}
\alias{mHypothal}
\alias{mh_data}
\alias{mh_expr}
\title{Mouse Hypothalamus Data}
\format{
\code{mh_expr} a gene expression matrix with normalised counts, where rows
indicate genes and columns indicate cells.
\code{mh_data} a data frame of cell metadata including cell IDs, sample IDs,
cell type annotations, and x-y coordinates of cells.
}
\source{
Molecular, Spatial and Functional Single-Cell Profiling of the
Hypothalamic Preoptic Region, \emph{Science}, 2018.
Webpage: \url{https://www.science.org/doi/10.1126/science.aau5324}
}
\usage{
data(mHypothal)
}
\description{
This dataset contains spatial transcriptomics data from 181 mouse
hypothalamus samples, 155 genes and a total of 1,027,080 cells. For running
the vignettes, we subset the data by selecting total 6000 cells from only 3
samples - Animal 1 Bregma -0.09 (2080 cells) and Animal 7 Bregmas 0.16
(1936 cells) and -0.09 (1984 cells), excluding cells that were annotated as
'ambiguous', and removed 20 genes that were assessed using a different
technology.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.cellName}
\alias{.cellName}
\title{Neighbour cell naming}
\usage{
.cellName(cell, Clust)
}
\arguments{
\item{cell}{a vector of neighbourhood cell indices. The cell indices indicate
the row number of cells in sample metadata.}

\item{Clust}{a data frame of initial cluster labels of each cell in the
sample.}
}
\value{
a data frame of cell IDs of neighbourhood cells.
}
\description{
A function to fetch cell IDs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.exp_kernel}
\alias{.exp_kernel}
\title{Exponential distribution weights}
\usage{
.exp_kernel(ed, NN, rate)
}
\arguments{
\item{ed}{a numeric vector of entropy values of all cell neighbourhoods.}

\item{NN}{an integer for the number of neighbourhood cells including the
index cell.}

\item{rate}{a numeric value for rate of exponential distribution.}
}
\value{
a matrix where the columns contain weights associated with the
entropy values.
}
\description{
A function to generate weights from an exponential distribution.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{.gauss_kernel}
\alias{.gauss_kernel}
\title{Gaussian distribution weights}
\usage{
.gauss_kernel(ed, NN, sd)
}
\arguments{
\item{ed}{a numeric vector of entropy values of all cell neighbourhoods.}

\item{NN}{an integer for the number of neighbourhood cells including the
index cell.}

\item{sd}{a numeric value for standard deviation of Gaussian distribution.}
}
\value{
a matrix where the columns contain weights associated with the
entropy values.
}
\description{
A function to generate weights from a Gaussian distribution.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mEmbryo2}
\alias{mEmbryo2}
\alias{me_data}
\alias{me_expr}
\title{Mouse Embryo Data}
\format{
\code{me_expr} a gene expression matrix with normalised counts, where rows
indicate genes and columns indicate cells.
\code{me_data} a data frame of cell metadata including cell IDs, sample IDs,
cell type annotations, and x-y coordinates of cells.
}
\source{
Integration of spatial and single-cell transcriptomic data elucidates
mouse organogenesis, \emph{Nature Biotechnology}, 2021.
Webpage: \url{https://www.nature.com/articles/s41587-021-01006-2}
}
\usage{
data(mEmbryo2)
}
\description{
This dataset contains spatial transcriptomics data from 3 mouse embryos, with
351 genes and a total of 57536 cells. For vignettes, we subset the data by
randomly selecting 5000 cells from embryo 2, excluding cells that were
annotated as 'low quality'.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adaptiveSmoothing.R
\name{adaptiveSmoothing}
\alias{adaptiveSmoothing}
\title{Adaptive smoothing}
\usage{
adaptiveSmoothing(spe, nnCells, NN = 30, kernel = "G", spread = 0.3)
}
\arguments{
\item{spe}{SpatialExperiment object containing neighbourhood entropy values
of each cell.}

\item{nnCells}{a character matrix of NN nearest neighbours - rows are index
cells and columns are their nearest neighbours ranging from closest to
farthest neighbour. For sort = TRUE, the neighbours belonging to the same
initial cluster as the index cell are moved closer to it.}

\item{NN}{an integer for the number of neighbouring cells the function
should consider. The value must be greater than or equal to 1. Default value
is 30.}

\item{kernel}{a character for type of distribution to be used. The two valid
values are "G" or "E" for Gaussian and exponential distributions,
respectively. Default value is "G".}

\item{spread}{a numeric value for distribution spread, represented by
standard deviation for Gaussian distribution and rate for exponential
distribution. Default value is 0.3 for Gaussian distribution. The recommended
value is 5 for exponential distribution.}
}
\value{
SpatialExperiment object including smoothed gene expression as an
additional assay.
}
\description{
A function to perform a weighted, adaptive smoothing of the gene expression
of each cell based on the heterogeneity of its neighbourhood. Heterogeneous
neighbourhoods are smoothed less with higher weights given to cells belonging
to same initial cluster as the index cell. Homogeneous neighbourhoods are
smoothed more with similar weights given to most cells.
}
\examples{
data(ClustSignal_example)

# requires matrix containing NN nearest neighbour cell labels (nnCells),
# generated using the neighbourDetect() function
spe <- clustSIGNAL::adaptiveSmoothing(spe, nnCells)
spe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entropyMeasure.R
\name{entropyMeasure}
\alias{entropyMeasure}
\title{Heterogeneity measure}
\usage{
entropyMeasure(spe, regXclust, threads = 1)
}
\arguments{
\item{spe}{SpatialExperiment object with initial cluster and subcluster
labels.}

\item{regXclust}{a list of vectors of each cell's neighbourhood composition
indicated by the proportion of initial subclusters it contains.}

\item{threads}{a numeric value for the number of CPU cores to be used for the
analysis. Default value set to 1.}
}
\value{
SpatialExperiment object with entropy values associated with each
cell.
}
\description{
A function to measure the heterogeneity of a cell's neighbourhood in terms of
entropy. Generally, homogeneous neighbourhoods have low entropy and
heterogeneous neighbourhoods have high entropy.
}
\examples{
data(ClustSignal_example)

# requires list containing cluster proportions of each region (regXclust),
# generated using the neighbourDetect() function
spe <- clustSIGNAL::entropyMeasure(spe, regXclust)
spe$entropy |> head()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initClustering.R
\name{p1_clustering}
\alias{p1_clustering}
\title{Initial non-spatial clustering}
\usage{
p1_clustering(
  spe,
  dimRed = "None",
  batch = FALSE,
  batch_by = "None",
  threads = 1,
  clustParams = list(clust_c = 0, subclust_c = 0, iter.max = 30, k = 10, cluster.fun =
    "louvain")
)
}
\arguments{
\item{spe}{a SpatialExperiment object containing spatial coordinates in
'spatialCoords' matrix and normalised gene expression in 'logcounts' assay.}

\item{dimRed}{a character indicating the name of the reduced dimensions to
use from the SpatialExperiment object (i.e., from reducedDimNames(spe)).
Default value is 'None'.}

\item{batch}{a logical parameter for whether to perform batch correction.
Default value is FALSE.}

\item{batch_by}{a character indicating name of colData(spe) column containing
the batch names. Default value is 'None'.}

\item{threads}{a numeric value for the number of CPU cores to be used for the
analysis. Default value set to 1.}

\item{clustParams}{a list of parameters for TwoStepParam clustering methods:
clust_c is the number of centers to use for clustering with KmeansParam. By
default set to 0, in which case the method uses either 5000 centers or 1/5th
of the total cells in the data as the number of centers, whichever is lower.
subclust_c is the number of centers to use for sub-clustering the initial
clusters with KmeansParam. The default value is 0, in which case the method
uses either 1 center or half of the total cells in the initial cluster as
the number of centers, whichever is higher.
iter.max is the maximum number of iterations to perform during clustering
and sub-clustering with KmeansParam. Default value is 30.
k is a numeric value indicating the k-value used for clustering and
sub-clustering with NNGraphParam. Default value is 10.
cluster.fun is a character indicating the graph clustering method used with
NNGraphParam. By default, the Louvain method is used.}
}
\value{
SpatialExperiment object with initial cluster and subcluster labels
of each cell.
}
\description{
A function to perform initial non-spatial clustering and sub-clustering of
normalised gene expression to generate 'initial clusters' and 'initial
subclusters'.
}
\examples{
data(ClustSignal_example)

spe <- clustSIGNAL::p1_clustering(spe, dimRed = "PCA")
spe$nsCluster |> head()
spe$initCluster |> head()
}
